from subprocess import call as subprocess_call
from subprocess import PIPE, Popen as subprocess_popen
from zshpower.utils.catch import current_user, current_shell
from zshpower.utils.check import is_tool, tools_requirements
from snakypy import printer
from snakypy.ansi import FG


def reload_zsh():
    subprocess_call("exec zsh", shell=True)


def change_shell():
    if current_shell()[0] != "zsh":
        try:
            subprocess_call(f"chsh -s $(which zsh) {current_user()}", shell=True)
        except KeyboardInterrupt:
            printer("Canceled by user", foreground=FG.WARNING)


def shell_command(cmd):
    p = subprocess_popen(
        cmd, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True
    )
    output, err = p.communicate()
    return output.replace("\n", ""), err


# # TODO: This may be for the future, but not now.
# #
# def bash_command(cmd):
#     subprocess_popen(["/bin/bash", "-c", cmd])

# def systemctl_is_active(service):
#     process = subprocess_popen(
#         ["systemctl", "is-active", service], stdout=PIPE, universal_newlines=True
#     )
#     output, err = process.communicate()
#     return output.replace("\n", ""), err



from zshpower.config import package

author = package.info["author"]["name"]
website_author = package.info["author"]["website"]
email = package.info["author"]["email"]

# TODO: A lógica e estrutura de como chamar a impressão do zshpower
# com shell script, vai dar resultado na questão performance.
# Quanto menos comando e verificação, mais rápido o retorno.
content = f"""#!/usr/bin/env zsh

## Information:
## ******************************************************************************
## Theme: {package.info["name"]}
## Description: Call script to start {package.info["name"]}.
## ******************************************************************************

## Export PATH local user
export PATH="$PATH:$HOME/.local/bin"

## Option using "add-zsh-hook"
function set_zshpower () {{
    state=$(which zshpower-shell > /dev/null 2>&1)
    if [ ! $? -ne 0 ]; then
      PROMPT="$(zshpower-shell prompt)"
      RPROMPT="$(zshpower-shell rprompt)"
    else
      PROMPT='%F{{green}}%n%f@%F{{magenta}}%m%f %F{{blue}}%B%~%b%f %# '
      RPROMPT='[%F{{yellow}}%?%f]'
    fi
}}
add-zsh-hook precmd set_zshpower

## Option not using "add-zsh-hook" - manual
# function zshpower_precmd() {{
#     state=$(which zshpower-shell > /dev/null 2>&1)
#     if [ ! $? -ne 0 ]; then
#       PROMPT="$(zshpower-shell prompt)"
#       RPROMPT="$(zshpower-shell rprompt)"
#     else
#       PROMPT='%F{{green}}%n%f@%F{{magenta}}%m%f %F{{blue}}%B%~%b%f %# '
#       RPROMPT='[%F{{yellow}}%?%f]'
#     fi
# }}
#
# function install_zshpower_precmd() {{
#   for s in "${{precmd_functions[@]}}"; do
#     if [ "$s" = "zshpower_precmd" ]; then
#       return
#     fi
#   done
#   precmd_functions+=(zshpower_precmd)
# }}
#
# if [ "$TERM" != "linux" ]; then
#    install_zshpower_precmd
# fi
"""



from os import getcwd
from subprocess import check_output
from os.path import isfile, join
from .lib.utils import Color, symbol_ssh, separator, element_spacing
from zshpower.utils.check import is_tool
from zshpower.utils.process import shell_command


def docker_status():
    cmd = """
    state=$(docker info > /dev/null 2>&1)
    if [[ $? -ne 0 ]]; then
        echo "disabled"
    fi
    """
    return shell_command(cmd)[0]


class Docker:
    def __init__(self, config):
        self.config = config
        self.dockerfile = join(getcwd(), "Dockerfile")
        self.docker_compose = join(getcwd(), "docker-compose.yml")
        self.docker_version_enable = config["docker"]["version"]["enable"]
        self.docker_version_micro_enable = config["docker"]["version"]["micro"][
            "enable"
        ]
        self.docker_symbol = symbol_ssh(config["docker"]["symbol"], "Dkr-")
        self.docker_color = config["docker"]["color"]
        self.docker_prefix_color = config["docker"]["prefix"]["color"]
        self.docker_prefix_text = element_spacing(config["docker"]["prefix"]["text"])

    def get_version(self, space_elem=" "):
        docker_v = (
            check_output(
                "docker version --format '{{.Server.Version}}'",
                shell=True,
                universal_newlines=True,
            )
            .replace("\n", "")
            .split(".")
        )
        if not self.docker_version_micro_enable:
            return f"{'.'.join(docker_v[:-1])}{space_elem}"
        return f"{'.'.join(docker_v)}{space_elem}"

    def __str__(self):
        if self.docker_version_enable:
            if not docker_status() == "disabled":
                if isfile(self.dockerfile) or isfile(self.docker_compose):
                    docker_prefix = (
                        f"{Color(self.docker_prefix_color)}"
                        f"{self.docker_prefix_text}{Color().NONE}"
                    )
                    docker_export = (
                        f"{separator(self.config)}{docker_prefix}"
                        f"{Color(self.docker_color)}"
                        f"{self.docker_symbol}{self.get_version()}{Color().NONE}"
                    )
                    return str(docker_export)
        return ""



from os import geteuid as os_geteuid
from os import environ, getcwd
from pathlib import Path
from .lib.utils import Color

# from .lib.utils import abspath_link
from .lib.utils import symbol_ssh, element_spacing


def shorten_path(file_path, length):
    return Path(*Path(file_path).parts[-length:])


class Directory:
    def __init__(self, config):
        self.username_enable = config["username"]["enable"]
        self.hostname_enable = config["hostname"]["enable"]
        self.directory_truncate_value = config["directory"]["truncation_length"]
        self.directory_symbol = symbol_ssh(config["directory"]["symbol"], "")
        self.directory_color = config["directory"]["color"]
        self.directory_prefix_color = config["directory"]["prefix"]["color"]
        self.directory_prefix_text = element_spacing(
            config["directory"]["prefix"]["text"]
        )

    def __str__(self, prefix="", space_elem=" "):
        if (
            self.username_enable
            or os_geteuid() == 0
            or self.hostname_enable
            or "SSH_CONNECTION" in environ
        ):
            prefix = (
                f"{Color(self.directory_prefix_color)}"
                f"{self.directory_prefix_text}{Color().NONE}"
            )

        if int(self.directory_truncate_value) < 0:
            self.directory_truncate_value = 0
        if int(self.directory_truncate_value) > 4:
            self.directory_truncate_value = 4

        # Old "abspath_link()"
        dir_truncate = str(shorten_path(getcwd(), self.directory_truncate_value))

        if dir_truncate.split("/")[-1:] == str(Path.home()).split("/")[-1:]:
            dir_truncate = "~"
        directory_export = (
            f"{prefix}{Color(self.directory_color)}{self.directory_symbol}"
            f"{dir_truncate}{space_elem}{Color().NONE}"
        )

        return str(directory_export)



from .lib.utils import Color
from os.path import isfile, isdir, join
from os import getcwd
from .lib.utils import symbol_ssh, separator, element_spacing
from zshpower.utils.process import shell_command


class NodeJs:
    def __init__(self, config):
        self.config = config
        self.package_json = join(getcwd(), "package.json")
        self.node_modules = join(getcwd(), "node_modules")
        self.node_version_enable = self.config["nodejs"]["version"]["enable"]
        self.node_version_micro_enable = self.config["nodejs"]["version"]["micro"][
            "enable"
        ]
        self.node_symbol = symbol_ssh(self.config["nodejs"]["symbol"], "node-")
        self.node_color = self.config["nodejs"]["color"]
        self.node_prefix_color = self.config["nodejs"]["prefix"]["color"]
        self.node_prefix_text = element_spacing(self.config["nodejs"]["prefix"]["text"])

    def get_version(self, space_elem=" "):
        tool = {
            "node": shell_command("node -v 2>/dev/null"),
            "nodejs": shell_command("nodejs -v 2>/dev/null"),
            "nodenv": shell_command("nodenv version-name"),
        }
        lst_version = ""
        if tool["nodenv"]:
            if tool["nodenv"][0] == "system" or tool["nodenv"][0] == "node":
                lst_version = tool["node"][0].split(".")
            else:
                lst_version = tool["nodenv"][0].split(".")
        elif tool["node"]:
            lst_version = tool["node"][0].split(".")
        elif tool["nodejs"]:
            lst_version = tool["nodejs"][0].split(".")

        if not self.node_version_micro_enable:
            return f"{'.'.join(lst_version[:-1])}{space_elem}".replace("v", "")
        return f"{'.'.join(lst_version)}{space_elem}".replace("v", "")

    def __str__(self):
        if self.node_version_enable:
            if isfile(self.package_json) or isdir(self.node_modules):
                node_prefix = (
                    f"{Color(self.node_prefix_color)}"
                    f"{self.node_prefix_text}{Color().NONE}"
                )
                node_export = (
                    f"{separator(self.config)}{node_prefix}"
                    f"{Color(self.node_color)}"
                    f"{self.node_symbol}{self.get_version()}{Color().NONE}"
                )
                return str(node_export)
        return ""



from tomlkit import parse as toml_parse
from tomlkit.exceptions import UnexpectedCharError, ParseError
from snakypy.file import read as snakypy_file_read
from snakypy.json import read as snakypy_json_read
from os import getcwd
from os.path import isfile, join, exists
from .lib.utils import Color, symbol_ssh, separator, element_spacing
from zshpower.utils.process import shell_command


class Configs:
    def __init__(self, config):
        self.package_enable = config["package"]["enable"]
        self.package_symbol = symbol_ssh(config["package"]["symbol"], "pkg-")
        self.package_color = config["package"]["color"]
        self.package_prefix_color = config["package"]["prefix"]["color"]
        self.package_prefix_text = element_spacing(config["package"]["prefix"]["text"])


class Package(Configs):
    def __init__(self, config):
        Configs.__init__(self, config)
        self.config = config
        self.package_file = join(getcwd(), "pyproject.toml")

    def get_version(self, space_elem=" "):
        if isfile(self.package_file):
            cmd = f"""< {self.package_file} grep "^version = *" | cut -d'"' -f2 | cut -d"'" -f2"""
            return f"{shell_command(cmd)[0]}{space_elem}"
        return ""

    def __str__(self):
        if self.package_enable and self.get_version() != "":
            package_prefix = (
                f"{Color(self.package_prefix_color)}"
                f"{self.package_prefix_text}{Color().NONE}"
            )
            return (
                f"{separator(self.config)}{package_prefix}"
                f"{Color(self.package_color)}"
                f"{self.package_symbol}{self.get_version()}{Color().NONE}"
            )
        return ""


class NodePackage(Package):
    def __init__(self, config):
        Package.__init__(self, config)
        self.package_file = join(getcwd(), "package.json")

    def get_version(self, space_elem=" "):
        if isfile(join(getcwd(), self.package_file)):
            parsed = snakypy_json_read(join(getcwd(), self.package_file))
            if "version" in parsed:
                return f"{parsed['version']}{space_elem}"
        return ""

    def __str__(self):
        return Package.__str__(self)


def get_package(config):
    files_project_py = (
        "manage.py",
        "setup.py",
        "__init__.py",
        "requirements.txt",
        "pyproject.toml",
    )
    for i in files_project_py:
        if exists(join(getcwd(), i)):
            return str(Package(config))
    return str(NodePackage(config))
